/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* alm
 *
 * Copyright (C) Seif Lotfy 2012 <seif@lotfy.com>
 * Copyright (C) Stefano Candori 2012 <stefano.candori@gmail.com>
 * Copyright (C) Manish Sinha 2012 <manishsinha@ubuntu.com>
 *
 * alm is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * alm is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */
 
using Gtk;
using Gee;
using GLib;
using Zeitgeist;

namespace Alm {

	public class ApplicationsWidget : Gtk.Box {

		private Blacklist blacklist_interface;

		private ApplicationBlacklist app_blacklist;
		private ApplicationsChooserDialog app_dialog;
		private ApplicationsTreeView app_treeview;
		private Gtk.Box container_box;

		public ApplicationsWidget (Blacklist blacklist_inter) {
			Object (orientation: Orientation.VERTICAL);
			
			this.blacklist_interface = blacklist_inter;

			this.app_blacklist = new ApplicationBlacklist (this.blacklist_interface);
			this.app_dialog = new ApplicationsChooserDialog (this.app_blacklist);
			this.set_up_ui ();
		}

		private void set_up_ui () {
			this.set_spacing(6);
			this.set_border_width (12);

			container_box = new Gtk.Box(Orientation.VERTICAL, 0);

			app_treeview = new ApplicationsTreeView(app_blacklist);
			this.app_blacklist.application_removed.connect(app_treeview.remove_app_from_view);
			this.app_blacklist.application_added.connect((app, event) => {
				this.add_application_to_view(app);
			});

			app_treeview.visible = true;
			var app_toolbar = new Toolbar();
			app_toolbar.toolbar_style = ToolbarStyle.ICONS;
			app_toolbar.icon_size = 1;
			app_toolbar.icon_size_set = true;
			app_toolbar.visible = true;

			var app_add = new ToolButton(null, "Add Application");
			app_add.set_icon_name("list-add-symbolic");
			app_add.clicked.connect(app_add_clicked);

			var app_remove = new ToolButton(null, "Remove Application");
			app_remove.set_icon_name("list-remove-symbolic");
			app_remove.clicked.connect(app_remove_clicked);

			app_toolbar.insert(app_add, 0);
			app_toolbar.insert(app_remove, 1);

			container_box.pack_start(app_treeview, true, true);
			container_box.pack_start(app_toolbar, false, false);

			var label = new Label(null);
			label.set_markup("<b>Do not log activity from the following applications:</b>");
			label.set_alignment(0, 0.5f);
			
			this.pack_start(label, false, false);
			this.pack_start(container_box, true, true);

			var app_list = this.app_blacklist.all_apps;
			this.populate_view(app_list);
		}

		public void app_add_clicked(ToolButton button) {
			int res = this.app_dialog.run();
			if(res == ResponseType.OK) {

			}
			this.app_dialog.hide();
		}

		public void app_remove_clicked(ToolButton button) {
			// Remove the selected app from the view
			var app = this.app_treeview.remove_selected_app();
			this.app_blacklist.unblock(app);
		}

		private void populate_view(HashSet<string> app_list) {
			foreach(string app in app_list)
				this.add_application_to_view(app);
		}

		public static string markup_for_app(DesktopAppInfo app_info) {
			var app_name = app_info.get_name ();
			var description = app_info.get_description();
			description = description != null? description: "No description available";

			return "<b>%s</b>\n%s".printf(GLib.Markup.escape_text(app_name), 
					GLib.Markup.escape_text(description));
		}

		private void add_application_to_view(string app) {
			DesktopAppInfo app_info = new DesktopAppInfo(app);
			if (app_info != null) {
				var pix = ApplicationsTreeView.get_pixbuf_from_gio_icon(app_info.get_icon());
				var markup = ApplicationsWidget.markup_for_app(app_info);
				TreeIter iter;
				this.app_treeview.liststore.append(out iter);
				this.app_treeview.liststore.set(iter, 0, markup, 1, pix, 2, app, -1);
			}
		}
	}

	public class ApplicationBlacklist { //: remote.GenericBlacklist
		private Blacklist blacklist_interface;
		
		public static string interpretation_prefix = "app-";

		private HashSet<string> all_blocked_apps;

		public signal void application_added(string app, Event ev);

		public signal void application_removed(string app, Event ev);

		public ApplicationBlacklist (Blacklist blacklist) {
			this.blacklist_interface = blacklist;
			this.blacklist_interface.template_added.connect(on_blacklist_added);
			this.blacklist_interface.template_removed.connect(on_blacklist_removed);

			this.get_blocked_apps();
			
		}

		public HashSet<string> all_apps {
			get {
				return all_blocked_apps;
			}
		}

		public void get_all_applications(ApplicationsChooserDialog dialog) {
			this.blacklist_interface.get_all_applications(dialog);
		}

		private HashSet<string> get_blocked_apps() {
			all_blocked_apps = new HashSet<string>();

			foreach(string key in blacklist_interface.all_templates.get_keys()) {
				if(key.has_prefix(interpretation_prefix))
				{
					var app = key.substring(4);
					all_blocked_apps.add(app);
				}
			}
			
			return all_blocked_apps;
		}

		private void on_blacklist_added (string blacklist_id, Event ev) {
			if(blacklist_id.has_prefix(interpretation_prefix))
			{
				string app = blacklist_id.substring(4);
				application_added(app, ev);
				if(!all_apps.contains(app))
					all_apps.add(app);
			}
		}

		private void on_blacklist_removed (string blacklist_id, Event ev) {
			if(blacklist_id.has_prefix(interpretation_prefix))
			{
				string app = blacklist_id.substring(4);
				application_removed(app, ev);
				if(all_apps.contains(app))
					all_apps.remove(app);
			}
		}

		public void block(string application) {
			/*Event ev = new Event();
			Subject sub = new Subject();
			sub.set_interpretation(interpretation);
			ev.add_subject(sub);

			blacklist_interface.add_template(
					this.get_name(interpretation), ev);

			if(!all_apps.contains(application))
					all_apps.add(application);*/
		}

		public void unblock(string application) {
			blacklist_interface.remove_template(
					"%s%s".printf(interpretation_prefix, application));

			if(all_apps.contains(application))
					all_apps.remove(application);
		}
	}

	private class ApplicationsTreeView : Gtk.Box {
	
		private ApplicationBlacklist app_blacklist;
		private static IconTheme icon_theme = IconTheme.get_default ();
		private ListStore store;
		private TreeView treeview;
		
		enum TreeViewCols
		{
			APP_NAME,
			ICON,
			DESKTOP_FILE,
			N_COLS
		}

		public ListStore liststore {
			get {
				return this.store;
			}
		}

		public ApplicationsTreeView (ApplicationBlacklist app_blacklist) {
			Object (orientation: Orientation.VERTICAL);
			
			this.app_blacklist = app_blacklist;
			this.store = new ListStore (TreeViewCols.N_COLS,
										typeof (string),
										typeof (Gdk.Pixbuf),
										typeof (string));
			this.treeview = new TreeView.with_model (this.store);
			this.treeview.set_headers_visible (false);
			this.treeview.set_rules_hint (true);
			this.set_up_ui ();
		}
		
		private void set_up_ui () {
			var column_pix_name = new TreeViewColumn ();
			column_pix_name.set_title ("Name");
			this.treeview.append_column (column_pix_name);
			var pix_rend = new CellRendererPixbuf ();
			column_pix_name.pack_start (pix_rend, false);
			column_pix_name.add_attribute (pix_rend, "pixbuf", 1);
			var name_rend = new CellRendererText ();
			name_rend.set_property ("ellipsize", Pango.EllipsizeMode.END);
			column_pix_name.pack_start (name_rend, true);
			column_pix_name.add_attribute (name_rend, "markup", 0);
			column_pix_name.set_resizable (true);
			
			var scroll = new ScrolledWindow (null, null);
			scroll.add (this.treeview);
			scroll.set_policy (PolicyType.NEVER, PolicyType.AUTOMATIC);
			scroll.set_shadow_type (ShadowType.IN);
			scroll.set_border_width (1);
			this.pack_start (scroll);
		}
		
		public static Gdk.Pixbuf? get_pixbuf_from_gio_icon (Icon icon) {
			Gdk.Pixbuf? pix = null;
			var icon_info = icon_theme.lookup_by_gicon (icon ,
															32,
															IconLookupFlags.FORCE_SVG);
			if (icon_info != null) {
				try {
					pix = icon_info.load_icon ();
				}
				catch (Error e){
					return null;
				}
			}
			
			return pix;
		}
		
		public void remove_app_from_view (string app, Event ev) {
			var model = this.treeview.get_model ();
			TreeIter iter;
			model.get_iter_first (out iter);
			while (true)
			{
				Value can_app_value;
				model.get_value(iter, 2, out can_app_value);
				string can_app = can_app_value.get_string();
				if(app == can_app)
				{
					this.store.remove(iter);
					break;
				}
				bool more_entires = model.iter_next(ref iter);
				if(!more_entires)
				{
					break;
				}
			}
		}
		
		public string? remove_selected_app() {
			var selection = this.treeview.get_selection();
			TreeModel model;
			TreeIter iter;
			string app = null;
			if(selection.get_selected(out model, out iter))
			{
				model.get(iter, 2, out app);
				this.liststore.remove(iter);
			}

			return app;
		}
	}

	private class AppSelectionTreeView : Gtk.Box {

		private ApplicationBlacklist app_blacklist;
		private ListStore store;
		private TreeView treeview;
		
		enum TreeViewCols
		{
			APP_NAME,
			ICON,
			DESKTOP_FILE,
			N_COLS
		}

		public AppSelectionTreeView (ApplicationBlacklist app_blacklist) {
			Object (orientation: Orientation.VERTICAL);
			
			this.app_blacklist = app_blacklist;
			this.store = new ListStore (TreeViewCols.N_COLS,
										typeof (string),
										typeof (Gdk.Pixbuf),
										typeof (string));
			this.treeview = new TreeView.with_model (this.store);
			this.treeview.set_headers_visible (false);
			this.treeview.set_rules_hint (true);
			this.set_up_ui ();
		}
		
		private void set_up_ui () {
			var column_pix_name = new TreeViewColumn ();
			column_pix_name.set_title ("Name");
			this.treeview.append_column (column_pix_name);
			var pix_rend = new CellRendererPixbuf ();
			column_pix_name.pack_start (pix_rend, false);
			column_pix_name.add_attribute (pix_rend, "pixbuf", 1);
			var name_rend = new CellRendererText ();
			name_rend.set_property ("ellipsize", Pango.EllipsizeMode.END);
			column_pix_name.pack_start (name_rend, true);
			column_pix_name.add_attribute (name_rend, "markup", 0);
			column_pix_name.set_resizable (true);
			
			var scroll = new ScrolledWindow (null, null);
			scroll.add (this.treeview);
			scroll.set_policy (PolicyType.NEVER, PolicyType.AUTOMATIC);
			scroll.set_shadow_type (ShadowType.IN);
			scroll.set_border_width (1);
			this.pack_start (scroll);
		}
	}
	
	public class ApplicationsChooserDialog : Gtk.Dialog {
		
		private ApplicationBlacklist app_blacklist;
		private AppSelectionTreeView treeview;
		
		public ApplicationsChooserDialog (ApplicationBlacklist app_blacklist) {
			this.app_blacklist = app_blacklist;
			this.set_title ("Select Application");
			this.set_destroy_with_parent (true);
			this.set_size_request (600, 400);
			this.set_skip_taskbar_hint (true);
			this.set_border_width (12);
			this.resizable = false;
			this.set_up_ui ();

			this.app_blacklist.get_all_applications(this);
		}
		
		public void handle_app_population(GLib.List<Event> all_apps) {

		}

		private void set_up_ui () {
			this.treeview = new AppSelectionTreeView (this.app_blacklist);
			var area = this.get_content_area () as Gtk.Box;
			area.pack_start(this.treeview);
			this.add_buttons (Stock.OK, ResponseType.OK, Stock.CANCEL, ResponseType.CANCEL);
			
			//this.treeview.populate_view ();
			this.treeview.show_all ();
		}
	}

}

