/* -*- Mode: vala; tab-width: 4; intend-tabs-mode: t -*- */
/* alm
 *
 * Copyright (C) Seif Lotfy 2012 <seif@lotfy.com>
 * Copyright (C) Manish Sinha 2012 <manishsinha@ubuntu.com>
 *
 * alm is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * alm is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

using Gtk;
using Gee;
using Zeitgeist;

namespace Alm {

	public class FileTypeBlacklist {
		private Blacklist blacklist_interface;
		private HashMap<string, CheckButton> checkboxes;

		public static string interpretation_prefix = "interpretation-";
		
		public FileTypeBlacklist (Blacklist blacklist_inter,
					HashMap<string, CheckButton> all_checkboxes) {
			blacklist_interface = blacklist_inter;
			checkboxes = all_checkboxes;
		}

		private string get_name(string interpretation) {
			var names = interpretation.split("#");
			var name = names[names.length-1].down();
			return "%s%s".printf(interpretation_prefix, name);
		}

		public void populate_file_types() {

			foreach(string key in blacklist_interface.all_templates.get_keys()) {
				if(key.has_prefix(this.interpretation_prefix))
				{
					var inter = blacklist_interface.all_templates[key].get_subject(0).get_interpretation ();
					if(checkboxes.has_key(inter))
						checkboxes.get(inter).active = true;
				}
			}
		}

		public void block(string interpretation) {
			Event ev = new Event();
			Subject sub = new Subject();
			sub.set_interpretation(interpretation);
			ev.add_subject(sub);

			blacklist_interface.add_template(
					this.get_name(interpretation), ev);
		}

		public void unblock(string interpretation) {
			blacklist_interface.remove_template(
					this.get_name(interpretation));
		}
	}

	public class PathBlacklist {
		private Blacklist blacklist_interface;
		
		public PathBlacklist (Blacklist blacklist_inter) {
			blacklist_interface = blacklist_inter;
		}
	}


	public class FilesWidget : Gtk.Box {

		private FileTypeBlacklist files_type_blacklist;
		private PathBlacklist path_blacklist;
		private TreeView folder_list;
		private ListStore folder_list_store;

		private HashMap<string, string> mime_dict;
		private HashMap<string, CheckButton> checkboxes;
		private HashSet<CheckButton> button_list;

		private Blacklist blacklist;
		private bool file_type_fire_signal = true;

		enum TreeViewCols
		{
			APP_NAME,
			ICON,
			DESKTOP_FILE,
			N_COLS
		}

		public FilesWidget (Blacklist blacklist_interface) {
			Object (orientation: Orientation.VERTICAL);

			this.blacklist = blacklist_interface;
			this.checkboxes = new HashMap<string, CheckButton>();
			this.button_list = new HashSet<CheckButton>();

			this.files_type_blacklist = new FileTypeBlacklist(blacklist_interface, 
												this.checkboxes);
			this.path_blacklist = new PathBlacklist(blacklist_interface);

			blacklist_interface.template_added.connect(
				(blacklist_id, blacklist_template) => {
					if(blacklist_id.has_prefix(FileTypeBlacklist.interpretation_prefix))
					{
						file_type_fire_signal = false;
						var inter = blacklist_template.get_subject(0).get_interpretation ();
						if(checkboxes.has_key(inter))
							checkboxes.get(inter).active = true;
						file_type_fire_signal = true;
					}
				});
			blacklist_interface.template_removed.connect(
				(blacklist_id, blacklist_template) => {
					if(blacklist_id.has_prefix(FileTypeBlacklist.interpretation_prefix))
					{
						file_type_fire_signal = false;
						var inter = blacklist_template.get_subject(0).get_interpretation ();
						if(checkboxes.has_key(inter))
							checkboxes.get(inter).active = false;
						file_type_fire_signal = true;
					}
				});

			mime_dict = new HashMap<string, string>();
			mime_dict.set("Audio", NFO_AUDIO);
	        mime_dict.set("Video", NFO_VIDEO);
	        mime_dict.set("Image", NFO_IMAGE);
	        mime_dict.set("Text", NFO_DOCUMENT);
	        mime_dict.set("Presentation", NFO_PRESENTATION);
	        mime_dict.set("Spreadsheet", NFO_SPREADSHEET);
	        mime_dict.set("Instant Messaging", NMO_IMMESSAGE);
	        mime_dict.set("E-mail", NMO_EMAIL);
	        mime_dict.set("Website", NFO_WEBSITE);
		
			this.setup_ui();
		}

		private void setup_ui() {

			this.set_border_width(12);

			var vbox_file_types = new Box(Orientation.VERTICAL, 6);
			this.pack_start(vbox_file_types, false, false);

			// Label for Checkbox list
			var file_type_label = new Label(null);
			file_type_label.set_markup("<b>Don't record activity for following type of files:</b>");
			file_type_label.set_alignment(0.0f, 0.5f);
			vbox_file_types.pack_start(file_type_label, false);

			// Checkbox Table
			var checkbox_table = new Table(3, 3, true);

			string[] keys = mime_dict.keys.to_array();
			for(int i=0,row=0,col=0; i < keys.length; i++, col++) {
			//foreach (var entry in mime_dict.entries) {
		        var check_button = new CheckButton.with_label (keys[i]);
				
				this.button_list.add(check_button);
				this.checkboxes.set(mime_dict[keys[i]], check_button);

				if(col > 2) {
					row++;
					col = col % 3;
				}

				checkbox_table.attach_defaults(check_button, col, col+1, row, row+1);
    		}
		
			vbox_file_types.pack_start(checkbox_table, false);

			this.files_type_blacklist.populate_file_types();

			foreach(CheckButton check_button in this.button_list)
			{
				check_button.toggled.connect(() => {
					if(file_type_fire_signal)
					{
						//debug(mime_dict[check_button.label]);
						file_type_fire_signal = false;
						if(check_button.active)
							files_type_blacklist.block(mime_dict[check_button.label]);
						else
							files_type_blacklist.unblock(mime_dict[check_button.label]);
						file_type_fire_signal = true;
					}
    			});
			}

			// Horizontal Line
			var hoz_line = new HSeparator();
			this.pack_start(hoz_line, false, false, 12);

			// Folders
			var vbox_folders = new Box(Orientation.VERTICAL, 6);
			this.pack_start(vbox_folders, true, true);

			// Label for Folder list list
			var folders_label = new Label(null);
			folders_label.set_markup("<b>Don't record activity in the following folders:</b>");
			folders_label.set_alignment(0.0f, 0.5f);
			vbox_folders.pack_start(folders_label, false);

			var file_chooser_vbox = new Box(Orientation.VERTICAL, 0);
			vbox_folders.pack_start(file_chooser_vbox, true, true);

			// Folder list TreeView
			this.folder_list_store = new ListStore (TreeViewCols.N_COLS,
													typeof(string));
			this.folder_list = new TreeView.with_model (this.folder_list_store);
			this.folder_list.set_headers_visible (false);
			this.folder_list.set_rules_hint (true);

			var scroll = new ScrolledWindow(null, null);
	        scroll.add(this.folder_list);
			file_chooser_vbox.pack_start(scroll, true, true);

			// Add Remove buttons
			var app_toolbar = new Toolbar();
			app_toolbar.toolbar_style = ToolbarStyle.ICONS;
			app_toolbar.icon_size = 1;
			app_toolbar.icon_size_set = true;
			app_toolbar.visible = true;

			var app_add = new ToolButton(null, "Add Application");
			app_add.set_icon_name("list-add-symbolic");

			var app_remove = new ToolButton(null, "Remove Application");
			app_remove.set_icon_name("list-remove-symbolic");

			app_toolbar.insert(app_add, 0);
			app_toolbar.insert(app_remove, 1);

			file_chooser_vbox.pack_start(app_toolbar, false, false);
		}
	}

}
